import unittest
from zetane import context as ztn

class TestRemap(unittest.TestCase):

    def setUp(self):
        self.zcontext = ztn.Context()
        self.zcontext.launch()

        # Raw data chart (PURPLE)
        zchart = self.zcontext.chartDEPRECATED()
        zchart.color(0.6, .5, .7)
        zchart.position(-5, 0, -10)
        zchart.add_border()
        zchart.smooth()

        # Remapped data chart (GREEN)
        zchart_remap = self.zcontext.chartDEPRECATED()
        zchart_remap.color(0.6, .7, .5)
        zchart_remap.position(-5, 0, -10)
        zchart_remap.smooth()

        self.zchart = zchart
        self.zchart_remap = zchart_remap

        # Raw data: range of possible values = [-5, 5]
        self.raw_points = np.array([0, 0, 0, 0, 1, 5, 3.5, -1, -2, 2.5, 1, 2, 3.5, 0, 0, 0])

    def tearDown(self):
        self.zcontext.clear_universe()

    def test_remap_1(self):
        # Ex.1: Normalize to (0, 1). I.e. remapping (input min, input max) to (0, 1)
        remapped_points = remap(self.raw_points, out_range=(0.0, 1.0))
        # Draw
        self.zchart.points(self.raw_points.tolist()).update()
        self.zchart_remap.points(remapped_points.tolist()).update()

    def test_remap_2(self):
        # Ex.2: Normalize to (-1, 1). I.e. remapping (input min, input max) to (-1, 1)
        remapped_points = remap(self.raw_points, out_range=(-1.0, 1.0))
        # Draw
        self.zchart.points(self.raw_points.tolist()).update()
        self.zchart_remap.points(remapped_points.tolist()).update()

    def test_remap_3(self):
        # Ex.3: Remap (-5, 5) to (-1, 1). Notice, input min/max doesn't affect the mapping
        remapped_points = remap(self.raw_points, in_range=(-5,5), out_range=(-1.0, 1.0))
        # Draw
        self.zchart.points(self.raw_points.tolist()).update()
        self.zchart_remap.points(remapped_points.tolist()).update()

    def test_remap_4(self):
        # Ex.4: Remap (-5, 5) to (1, -1). Notice, this valid remap flips the y values
        remapped_points = remap(self.raw_points, in_range=(-5,5), out_range=(1.0, -1.0))
        # Draw
        self.zchart.points(self.raw_points.tolist()).update()
        self.zchart_remap.points(remapped_points.tolist()).update()

    def test_remap_5(self):
        # Ex.5: Remap (-1, 1) to (-1, 1). Notice, clamps raw values outside of in_range
        remapped_points = remap(self.raw_points, in_range=(-1,1), out_range=(-1.0, 1.0), clamp=True)
        # Draw
        self.zchart.points(self.raw_points.tolist()).update()
        self.zchart_remap.points(remapped_points.tolist()).update()
